__title__ = 'minotaur'
__description__ = 'Pythonic, asynchronous, inotify interface.'
__url__ = 'https://github.com/giannitedesco/minotaur'
__author__ = 'Gianni Tedesco'
__author_email__ = 'gianni@scaramanga.co.uk'
__copyright__ = 'Copyright 2020 Gianni Tedesco'
__license__ = 'Apache 2.0'
__version__ = '0.0.2'
